@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/address_group.c
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.7.1   2009/11/11
+ * Version: 1.7.3   2011/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -27,7 +27,7 @@ LIST_HEAD(ccs_address_group_list);
 struct ccs_address_group *ccs_get_address_group(const char *group_name)
 {
 	struct ccs_address_group *entry = NULL;
-	struct ccs_address_group *group;
+	struct ccs_address_group *group = NULL;
 	const struct ccs_path_info *saved_group_name;
 	int error = -ENOMEM;
 	if (!ccs_is_correct_path(group_name, 0, 0, 0) ||
@@ -36,8 +36,9 @@ struct ccs_address_group *ccs_get_address_group(const char *group_name)
 	saved_group_name = ccs_get_name(group_name);
 	if (!saved_group_name)
 		return NULL;
-	entry = kzalloc(sizeof(*entry), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	entry = kzalloc(sizeof(*entry), CCS_GFP_FLAGS);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry_rcu(group, &ccs_address_group_list, list) {
 		if (saved_group_name != group->group_name)
 			continue;
@@ -56,6 +57,7 @@ struct ccs_address_group *ccs_get_address_group(const char *group_name)
 		error = 0;
 	}
 	mutex_unlock(&ccs_policy_lock);
+ out:
 	ccs_put_name(saved_group_name);
 	kfree(entry);
 	return !error ? group : NULL;
@@ -72,7 +74,6 @@ struct ccs_address_group *ccs_get_address_group(const char *group_name)
 int ccs_write_address_group_policy(char *data, const bool is_delete)
 {
 	struct ccs_address_group *group;
-	struct ccs_address_group_member *entry = NULL;
 	struct ccs_address_group_member *member;
 	struct ccs_address_group_member e = { };
 	int error = is_delete ? -ENOENT : -ENOMEM;
@@ -101,21 +102,22 @@ int ccs_write_address_group_policy(char *data, const bool is_delete)
 	default:
 		goto out;
 	}
-	if (!is_delete)
-		entry = kmalloc(sizeof(e), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry_rcu(member, &group->member_list, list) {
-		if (ccs_memcmp(member, &e, offsetof(typeof(e), is_ipv6),
-			       sizeof(e)))
+		if (!ccs_is_same_address_group_member(member, &e))
 			continue;
 		member->is_deleted = is_delete;
 		error = 0;
 		break;
 	}
-	if (!is_delete && error && ccs_commit_ok(entry, &e, sizeof(e))) {
-		list_add_tail_rcu(&entry->list, &group->member_list);
-		entry = NULL;
-		error = 0;
+	if (!is_delete && error) {
+		struct ccs_address_group_member *entry =
+			ccs_commit_ok(&e, sizeof(e));
+		if (entry) {
+			list_add_tail_rcu(&entry->list, &group->member_list);
+			error = 0;
+		}
 	}
 	mutex_unlock(&ccs_policy_lock);
  out:
@@ -124,7 +126,6 @@ int ccs_write_address_group_policy(char *data, const bool is_delete)
 		ccs_put_ipv6_address(e.max.ipv6);
 	}
 	ccs_put_address_group(group);
-	kfree(entry);
 	return error;
 }
 
